/*
 * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
//The example is developed under KDS3.2.0 and TRK-KEA128 board by XiangJun Rong
//The example code demonstrates how to trigger ADC at the center of high logic of  bottom PWM signal
//the PWM signal is generated by FTM2
//The KEA128 is designed to enable the FTM2 to generate signal to trigger ADC so that the sampling instant is at the center of PWM high logic of bottom PWM.
//Conclusion: connect the PTB2 and PTC0 pin to scope you can see that the PTB2 toggle at the center of  high logic of PTC0 PWM signal
//SIM_SOPT0 register provide the mechanism to control the sampling instant of ADC.
#include "SKEAZ1284.h"

void PWMOutput_CenterAlignment(void);
void enableADCFTMClock(void);
void AdcInit(void);
void ADC_IRQHandler(void);
void FRM2_TriggerSignal(void);
void GPIOInit(void);
static int i = 0;
unsigned int sample[8];

void enableADCFTMClock(void)
{
	//enable ADC, FTM0/1/2 clock
	SIM_SCGC|=0x200000E0;

}


void PWMOutput_CenterAlignment(void)
{
	FTM2_CONF=0xC0; //set up BDM in 11
	FTM2_FMS=0x00; //clear the WPEN so that WPDIS is set in FTM0_MODE reg
	FTM2_MODE|=0x05; //enable write the FTM CnV register
	FTM2_MOD=1000;
	FTM2_C0SC=0x28; ////center-alignment, PWM begins with High
	FTM2_C1SC=0x28; //PWM waveform is high-low-high
	FTM2_COMBINE=0x02; //complementary mode for CH0&CH1 of FTM0
	FTM2_COMBINE|=0x10; // dead timer insertion enabled in complementary mode for //CH0&CH1 of FTM0
	FTM2_DEADTIME=0x1F; //dead time is 16 system clock cycles
	FTM2_C1V=500;
	FTM2_C0V=500;
	FTM2_CNTIN=0x00;
	FTM2_C2SC=0x28;
	FTM2_C3SC=0x28;
	FTM2_COMBINE|=0x0200;
	FTM2_COMBINE|=0x1000;
	FTM2_C3V=250;
	FTM2_C2V=250;
	FTM2_SC=0x68;
}
//Pin assignment
//PTC0         FTM2_CH0 ALT2
//PTC1         FTM2_CH1 ALT2
//PTC2         FTM2_CH2 ALT2
//PTC3         FTM2_CH3 ALT2
//PTB4         FTM2_CH4 ALT2
//PTB5         FTM2_CH5 ALT2


void FRM2_TriggerSignal(void)
{
	FTM2_EXTTRIG=0x40; //enable FTM2_CNTIN register triggering
	SIM_SOPT0|=0x00200000; //set the ADHWT as 3b'010
	__asm("nop");
}
//ADC interrupt index is 15
void AdcInit(void)
{

	ADC_SC2=0x40; //hardware triggered,
	ADC_SC3=0x03; //not continuous conversion
	ADC_SC1=0x54; //interrupt enable and select ADC1_DM1 channel
//set that FTM0 to tri>gger ADC1
	NVIC->IP[3]&=0x3FFFFFFF; //setting interrupt of ADC1
	NVIC->ICPR[0]|=1<<15; //clear the pending register of interrupt source 58
	NVIC->ISER[0]|=1<<15; //set the interrupt source of ADC1
	__asm("cpsie i"); //interrupt enable
}


void ADC_IRQHandler(void)
{
	GPIOA_PTOR|=0x0400; //toggle indicator PTB2 pin
	sample[0]=ADC_R;
	__asm("nop");
//ADC1_SC1A=0x14; //interrupt enable and select ADC1_DM1 channel
}
//set PTB2 as GPIO output mode
void GPIOInit(void)
{
	FGPIOA_PDDR|=0x0400;
	FGPIOA_PIDR|=0x0400;
	FGPIOA_PTOR|=0x0400; //toggle indicator
	FGPIOA_PTOR=0x0400; //toggle indicator
	FGPIOA_PTOR=0x0400; //toggle indicator
	FGPIOA_PTOR=0x0400; //toggle indicator
	FGPIOA_PTOR=0x0400; //toggle indicator
	FGPIOA_PTOR=0x0400; //toggle indicator
	FGPIOA_PTOR=0x0400; //toggle indicator
}

int main(void)
{

    /* Write your code here */
	//PTC0/1/2/3 are PWM output pins
	SIM_PINSEL1&=0xFF00;
	enableADCFTMClock();
	GPIOInit();
	PWMOutput_CenterAlignment();
	AdcInit();
	FRM2_TriggerSignal();
    /* This for loop should be replaced. By default this loop allows a single stepping. */
    for (;;) {
        i++;
    }
    /* Never leave main */
    return 0;
}
////////////////////////////////////////////////////////////////////////////////
// EOF
////////////////////////////////////////////////////////////////////////////////
